@@ -16,7 +16,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import argparse
-import asyncio
 from dataclasses import dataclass
 import logging
 import re
@@ -66,65 +65,72 @@ class TransitionalPackage:
         return {'from_name': self.from_name, 'to_expr': self.to_expr}
 
 
-async def find_reverse_dependencies(udd, package):
+def find_reverse_dependencies(udd, package):
     by_source: Dict[str, Set[str]] = {}
     fields = ['recommends', 'depends', 'pre_depends', 'enhances',
               'suggests', 'provides']
     query = (
         "SELECT source, package, " + ", ".join(fields) +
         " FROM packages WHERE " + " OR ".join([
-            f"{field} LIKE CONCAT('%', $1::text, '%')" for field in fields]))
-    for row in await udd.fetch(query, package):
-        for relname in fields:
-            parsed = parse_relations(row[relname] or '')
-            for option in parsed:
-                for rel in option:
-                    if rel['name'] == package:
-                        by_source.setdefault(
-                            row['source'], set()).add(row['package'])
+            f"{field} LIKE CONCAT('%%', %s::text, '%%')" for field in fields]))
+    with udd.cursor() as cursor:
+        cursor.execute(query, (package, ))
+        for row in cursor:
+            for relname in fields:
+                parsed = parse_relations(row[relname] or '')
+                for option in parsed:
+                    for rel in option:
+                        if rel['name'] == package:
+                            by_source.setdefault(
+                                row['source'], set()).add(row['package'])
 
     fields = ["build_depends", "build_depends_indep", "build_depends_arch",
               "build_conflicts", "build_conflicts_indep",
               "build_conflicts_arch"]
     query = ("SELECT source, " + ", ".join(fields) +
              " FROM sources WHERE " + " OR ".join([
-                 f"{field} LIKE CONCAT('%', $1::text, '%')"
+                 f"{field} LIKE CONCAT('%%', %s::text, '%%')"
                  for field in fields]))
-    for row in await udd.fetch(query, package):
-        for relname in fields:
-            parsed = parse_relations(row[relname] or '')
-            for option in parsed:
-                for rel in option:
-                    if rel['name'] == package:
-                        by_source.setdefault(row['source'], set())
+    with udd.cursor() as cursor:
+        cursor.execute(query, (package, ))
+        for row in cursor:
+            for relname in fields:
+                parsed = parse_relations(row[relname] or '')
+                for option in parsed:
+                    for rel in option:
+                        if rel['name'] == package:
+                            by_source.setdefault(row['source'], set())
     return by_source
 
 
-async def find_dummy_transitional_packages(udd, release):
+def find_dummy_transitional_packages(udd, release):
     ret = {}
     query = """\
 SELECT package, description, depends FROM packages
-WHERE release = $1 AND description LIKE '%transitional%'"""
-    for row in await udd.fetch(query, release):
-        for regex in REGEXES:
-            if re.fullmatch(regex, row[1]):
-                break
-        else:
-            logging.debug(
-                'Unknown syntax for dummy package description: %r', row[1])
-        if row[2] is None:
-            logging.debug('no replacement for %s', row[0])
-        else:
-            depends = parse_relations(row[2])
-            if len(depends) != 1:
+WHERE release = %s AND description LIKE '%%transitional%%'"""
+    with udd.cursor() as cursor:
+        cursor.execute(query, (release, ))
+        for row in cursor:
+            for regex in REGEXES:
+                if re.fullmatch(regex, row[1]):
+                    break
+            else:
                 logging.debug(
-                    'no single transition target for %s: %r', row[0], row[2])
-        ret[row[0]] = TransitionalPackage(
-            from_name=row[0], to_expr=row[2])  # type: ignore
-    return ret
+                    'Unknown syntax for dummy package description: %r', row[1])
+            if row[2] is None:
+                logging.debug('no replacement for %s', row[0])
+            else:
+                depends = parse_relations(row[2])
+                if len(depends) != 1:
+                    logging.debug(
+                        'no single transition target for %s: %r',
+                        row[0], row[2])
+            ret[row[0]] = TransitionalPackage(
+                from_name=row[0], to_expr=row[2])  # type: ignore
+        return ret
 
 
-async def main(argv=None):
+def main(argv=None):
     from .udd import connect_udd_mirror
     parser = argparse.ArgumentParser(prog="candidates")
     parser.add_argument("--release", type=str, default="sid")
@@ -134,23 +140,23 @@ async def main(argv=None):
 
     logging.basicConfig(level=logging.INFO, format='%(message)s')
 
-    async with await connect_udd_mirror() as udd:
-        transitions = await find_dummy_transitional_packages(udd, args.release)
+    udd = connect_udd_mirror()
+    transitions = find_dummy_transitional_packages(udd, args.release)
 
-        if args.list_transitional_dummy:
-            yaml.dump(
-                [transition.json() for transition in transitions.values()],
-                sys.stdout)
-        elif args.list_uses_transitional_dummy:
-            for dep in transitions:
-                by_source = await find_reverse_dependencies(udd, dep)
-                for source, binaries in by_source.items():
-                    for binary in binaries:
-                        print(f'{source} / {binary} / {dep}')
-        else:
-            parser.print_usage()
-            return 1
+    if args.list_transitional_dummy:
+        yaml.dump(
+            [transition.json() for transition in transitions.values()],
+            sys.stdout)
+    elif args.list_uses_transitional_dummy:
+        for dep in transitions:
+            by_source = find_reverse_dependencies(udd, dep)
+            for source, binaries in by_source.items():
+                for binary in binaries:
+                    print(f'{source} / {binary} / {dep}')
+    else:
+        parser.print_usage()
+        return 1
 
 
 if __name__ == '__main__':
-    sys.exit(asyncio.run(main(sys.argv[1:])))
+    sys.exit(main(sys.argv[1:]))
